@@ -1,16 +1,16 @@
 define(
 //begin v1.x content
 {
-	"AUD_displayName": "dólar australiano",
-	"AUD_symbol": "AU$",
-	"CAD_displayName": "dólar canadiense",
-	"CHF_displayName": "franco suizo",
-	"CNY_displayName": "yuan chino",
-	"EUR_displayName": "euro",
-	"GBP_displayName": "libra esterlina británica",
 	"HKD_displayName": "dólar de Hong Kong",
+	"CHF_displayName": "franco suizo",
+	"CAD_displayName": "dólar canadiense",
+	"CNY_displayName": "yuan chino",
+	"AUD_displayName": "dólar australiano",
 	"JPY_displayName": "yen japonés",
-	"USD_displayName": "dólar estadounidense"
+	"USD_displayName": "dólar estadounidense",
+	"GBP_displayName": "libra esterlina británica",
+	"AUD_symbol": "AU$",
+	"EUR_displayName": "euro"
 }
 //end v1.x content
 );
\ No newline at end of file
