@@ -1,44 +1,50 @@
 define([
-"dojo/_base/declare", 
-"dojo/_base/lang", 
-"dojo/_base/array", 
-"dojo/_base/window", 
-"dojo/_base/event",
-"dojo/_base/html", 
-"dojo/_base/sniff", 
-"dojo/query", 
-"dojo/dom", 
-"dojo/dom-style",
-"dojo/dom-construct", 
-"dojo/on", 
-"dojo/date", 
-"dojo/date/locale", 
-"dijit/_WidgetBase",  
-"dojox/widget/_Invalidating", 
-"dojox/widget/Selection", 
-"dojox/calendar/time",
-"./StoreMixin"],
+	"dojo/_base/declare",
+	"dojo/_base/lang",
+	"dojo/_base/array",
+	"dojo/_base/window",
+	"dojo/_base/event",
+	"dojo/_base/html",
+	"dojo/sniff",
+	"dojo/query",
+	"dojo/dom",
+	"dojo/dom-style",
+	"dojo/dom-class",
+	"dojo/dom-construct",
+	"dojo/dom-geometry",
+	"dojo/on",
+	"dojo/date",
+	"dojo/date/locale",
+	"dojo/when",
+	"dijit/_WidgetBase",
+	"dojox/widget/_Invalidating",
+	"dojox/widget/Selection",
+	"dojox/calendar/time",
+	"./StoreMixin"],
 
-function(
-	declare, 
-	lang, 
-	arr, 
-	win, 
-	event, 
-	html, 
-	has, 
-	query, 
-	dom, 
-	domStyle,
-	domConstruct, 
-	on, 
-	date, 
-	locale, 
-	_WidgetBase, 
-	_Invalidating, 
-	Selection, 
-	timeUtil, 
-	StoreMixin){
+	function(
+		declare,
+		lang,
+		arr,
+		win,
+		event,
+		html,
+		has,
+		query,
+		dom,
+		domStyle,
+		domClass,
+		domConstruct,
+		domGeometry,
+		on,
+		date,
+		locale,
+		when,
+		_WidgetBase,
+		_Invalidating,
+		Selection,
+		timeUtil,
+		StoreMixin){
 	
 	/*=====
 	var __GridClickEventArgs = {
@@ -69,7 +75,9 @@ function(
 		// summary:
 		//		An item editing event.
 		// item: Object
-		//		The date item that is being edited.
+		//		The render item that is being edited. Set/get the startTime and/or endTime properties to customize editing behavior.
+		// storeItem: Object
+		//		The real data from the store. DO NOT change properties, but you may use properties of this item in the editing behavior logic.
 		// editKind: String
 		//		Kind of edit: "resizeBoth", "resizeStart", "resizeEnd" or "move".
 		// dates: Date[]
@@ -93,6 +101,18 @@ function(
 	};
 	=====*/
 	
+	/*=====
+	var __rendererLifecycleEventArgs = {
+		// summary:
+		//		An renderer lifecycle event.
+		// renderer: Object
+		//		The renderer.		
+		// source: dojox/calendar/ViewBase
+		//		The view where the event occurred.
+		// item:Object?
+		//		The item that will be displayed by the renderer for the "rendererCreated" and "rendererReused" events. 
+	};
+	=====*/
 
 	return declare("dojox.calendar.ViewBase", [_WidgetBase, StoreMixin, _Invalidating, Selection], {
 		
@@ -125,13 +145,14 @@ function(
 		//		Optional function to format the time of day of the item renderers.
 		//		The function takes the date and render data object as arguments and returns a String.
 		formatItemTimeFunc: null,
+		
+		_cssDays: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
 				
 		_getFormatItemTimeFuncAttr: function(){
 			if(this.owner != null){
 				return this.owner.get("formatItemTimeFunc");
-			}else{
-				return this.formatItemTimeFunc;
 			}
+			return this.formatItemTimeFunc;			
 		},
 		
 		// The listeners added by the view itself.
@@ -160,7 +181,7 @@ function(
 			while(this.rendererList.length > 0){
 				this._destroyRenderer(this.rendererList.pop());
 			}			
-			for(kind in this._rendererPool){				
+			for(var kind in this._rendererPool){
 				var pool = this._rendererPool[kind];
 				if(pool){
 					while(pool.length > 0){
@@ -176,6 +197,27 @@ function(
 			this.inherited(arguments);
 		},
 		
+		
+		resize: function(changeSize){
+			// summary:
+			//		Function to call when the view is resized. 
+			//		If the view is in a Dijit container or in a Dojo mobile container, it will be automatically called.
+			//		On other use cases, this method must called when the window is resized and/or when the orientation has changed.
+			if(changeSize){
+				domGeometry.setMarginBox(this.domNode, changeSize);
+			}
+		},
+		
+		_getTopOwner: function(){
+			// summary:
+			//		Returns the top owner: the calendar or the parent view.
+			var p = this;
+			while(p.owner != undefined){
+				p = p.owner;
+			}
+			return p;
+		},
+		
 		_createRenderData: function(){
 			// summary:
 			//		Creates the object that contains all the data needed to render this widget.
@@ -208,7 +250,9 @@ function(
 					}				
 			
 					var tNode = win.doc.createElement("span");
-					this.applyTextDir(tNode, text);					
+					if(has("dojo-bidi")){
+						this.applyTextDir(tNode, text);
+					}
 					
 					if(allowHTML){
 						tNode.innerHTML = text;
@@ -233,7 +277,7 @@ function(
 			
 			while(node != ancestor && node != document){
 				
-				if(dojo.hasClass(node, className)){
+				if(domClass.contains(node, className)){
 					return true;
 				}
 				
@@ -559,11 +603,7 @@ function(
 				return false;
 			} 			
 			
-			if(cal.compare(item.endTime, rd.endTime) == 1){
-				return false;
-			}
-			
-			return true;
+			return cal.compare(item.endTime, rd.endTime) != 1;
 		},
 		
 		_ensureItemInView: function(item){
@@ -639,9 +679,9 @@ function(
 				// reset
 				if(this._domScroll !== undefined){
 					if(this._domScroll){
-						domStyle.set(this.sheetContainer, this._cssPrefix+"transform", "translateY(-"+pos+"px)");
+						domStyle.set(this.sheetContainer, this._cssPrefix+"transform", "translateY(0px)");
 					}else{
-						this.scrollContainer.scrollTop = 0
+						this.scrollContainer.scrollTop = 0;
 					}
 				}
 				
@@ -706,7 +746,8 @@ function(
 			}
 			if(has("opera")){
 				return "-o-";
-			}			
+			}
+            return "";
 		},				
 		
 		_setScrollPosition: function(pos){
@@ -730,8 +771,18 @@ function(
 				}
 			}
 			
+			var containerSize = domGeometry.getMarginBox(this.scrollContainer);
+			var sheetSize = domGeometry.getMarginBox(this.sheetContainer);
+			var max = sheetSize.h - containerSize.h;
+			
+			if(pos < 0){
+				pos = 0;
+			}else if(pos > max){
+				pos = max;
+			}
+			
 			this._scrollPos = pos;
-							
+												
 			if(this._domScroll){				
 				this.scrollContainer.scrollTop = pos;				
 			}else{			
@@ -809,10 +860,6 @@ function(
 			this._layoutRenderers(this.renderData);
 		},
 		
-		resize: function(){
-			//this.invalidateRendering();
-		},
-
 		////////////////////////////////////////////////////////
 		//
 		// Layout
@@ -838,7 +885,6 @@ function(
 				};
 			}
 			
-			var numLanesPerInt;
 			var lanes = [];
 
 			for(var i=0; i<layoutItems.length; i++){
@@ -846,7 +892,7 @@ function(
 				this._layoutPass1(layoutItem, lanes);
 			}
 
-			var addedPassRes;
+			var addedPassRes = null;
 			if(func){
 				addedPassRes = lang.hitch(this, func)(lanes);
 			}
@@ -1079,7 +1125,7 @@ function(
 		_defaultItemToRendererKindFunc:function(item){
 			// tags:
 			//		private
-			return null
+			return null;
 		},
 
 		_createRenderer: function(item, kind, rendererClass, cssClass){			
@@ -1097,7 +1143,7 @@ function(
 						
 			if(item != null && kind != null && rendererClass != null){
 				
-				var res, renderer;
+				var res=null, renderer=null;
 				
 				var pool = this.rendererPool[kind];
 				
@@ -1108,28 +1154,19 @@ function(
 				if (res == null){
 
 					renderer = new rendererClass;
-
-					// the container allow to lay out the renderer
-					// this is important for styling (in box model 
-					// content size does take into account border)
-					var container = domConstruct.create("div");
-
-					// The DOM object that will contain the event renderer
-					container.className = "dojoxCalendarEventContainer "+ cssClass ;
-					container.appendChild(renderer.domNode);
-					
+									
 					res = {
 						renderer: renderer,
 						container: renderer.domNode,
 						kind: kind
 					};
 
-					this.onRendererCreated(res);
+					this._onRendererCreated({renderer:res, source:this, item:item});
 					
 				} else {
 					renderer = res.renderer; 
 					
-					this.onRendererReused(renderer);
+					this._onRendererReused({renderer:renderer, source:this, item:item});
 				}
 				
 				renderer.owner = this;
@@ -1146,41 +1183,77 @@ function(
 				return res;	
 			}
 			return null;
-		},	
-						
-		onRendererCreated: function(renderer){
+		},
+		
+		_onRendererCreated: function(e){
+			if(e.source == this){
+				this.onRendererCreated(e);
+			}
+			if(this.owner != null){
+				this.owner._onRendererCreated(e);
+			}
+		},
+		
+		onRendererCreated: function(e){
 			// summary:
 			//		Event dispatched when an item renderer has been created.
-			// renderer: dojox/calendar/_RendererMixin
-			//		The renderer created.
+			// e: __rendererLifecycleEventArgs
+			//		The renderer lifecycle event.
 			// tags:
 			//		callback
 		},	
 		
-		onRendererRecycled: function(renderer){
+		_onRendererRecycled: function(e){
+			if(e.source == this){
+				this.onRendererRecycled(e);
+			}
+			if(this.owner != null){
+				this.owner._onRendererRecycled(e);
+			}
+		},
+		
+		onRendererRecycled: function(e){
 			// summary:
 			//		Event dispatched when an item renderer has been recycled.
-			// renderer: dojox/calendar/_RendererMixin
-			//		The renderer recycled.
+			// e: __rendererLifecycleEventArgs
+			//		The renderer lifecycle event.
 			// tags:
 			//		callback
 
 		},
+						
+		_onRendererReused: function(e){
+			if(e.source == this){
+				this.onRendererReused(e);
+			}
+			if(this.owner != null){
+				this.owner._onRendererReused(e);
+			}
+		},
 		
-		onRendererReused: function(renderer){
+		onRendererReused: function(e){
 			// summary:
 			//		Event dispatched when an item renderer that was recycled is reused.
-			// renderer: dojox/calendar/_RendererMixin
-			//		The renderer reused.
+			// e: __rendererLifecycleEventArgs
+			//		The renderer lifecycle event.
 			// tags:
 			//		callback
 		},
 		
-		onRendererDestroyed: function(renderer){
+		_onRendererDestroyed: function(e){
+			if(e.source == this){
+				this.onRendererDestroyed(e);
+			}
+			if(this.owner != null){
+				this.owner._onRendererDestroyed(e);
+			}
+		},
+		
+		onRendererDestroyed: function(e){
 			// summary:
 			//		Event dispatched when an item renderer is destroyed.
-			// renderer: dojox/calendar/_RendererMixin
-			//		The renderer destroyed.
+			// e: __rendererLifecycleEventArgs
+			//		The renderer lifecycle event.
 			// tags:
 			//		callback
 		},
@@ -1191,7 +1264,7 @@ function(
 
 			this.onRenderersLayoutDone(view);
 			if(this.owner != null){
-				this.owner.onRenderersLayoutDone(view);
+				this.owner._onRenderersLayoutDone(view);
 			}				
 		},
 									
@@ -1210,7 +1283,7 @@ function(
 			// tags:
 			//		protected			
 								
-			this.onRendererRecycled(renderer);
+			this._onRendererRecycled({renderer:renderer, source:this});
 			
 			var pool = this.rendererPool[renderer.kind];
 			
@@ -1237,13 +1310,9 @@ function(
 			//		The item renderer to destroy.
 			// tags:
 			//		protected
-			this.onRendererDestroyed(renderer);
+			this._onRendererDestroyed({renderer:renderer, source:this});
 			
-			var ir = renderer.renderer;
-			
-			arr.forEach(ir.__handles, function(handle){
-				handle.remove();
-			});				
+			var ir = renderer.renderer;		
 			
 			if(ir["destroy"]){
 				ir.destroy();
@@ -1282,7 +1351,7 @@ function(
 			// summary:
 			//		Update the moveEnabled and resizeEnabled properties of a renderer according to its event current editing state.
 			// item: Object
-			//		The event data item.
+			//		The store data item.
 			// renderer: dojox/calendar/_RendererMixin
 			//		The item renderer.
 			// tags:
@@ -1339,7 +1408,7 @@ function(
 				var selected = this.isItemSelected(item);
 				var hovered = this.isItemHovered(item);
 				var edited = this.isItemBeingEdited(item);
-				var focused = this.showFocus ? this.isItemFocused(item) : false;
+				var focused = this.showFocus ? this.isItemFocused(item) : false;				
 				
 				for(var j = 0; j < list.length; j++){
 					
@@ -1348,6 +1417,7 @@ function(
 					renderer.set("selected", selected);
 					renderer.set("edited", edited);
 					renderer.set("focused", focused);
+					renderer.set("storeState", this.getItemStoreState(item));
 					
 					this.applyRendererZIndex(item, list[j], hovered, selected, edited, focused);
 					
@@ -1419,7 +1489,7 @@ function(
 				this.updateRenderers([old, this.hoveredItem], true);
 				
 				if(item && renderer){
-					this._updateEditingCapabilities(item, renderer);
+					this._updateEditingCapabilities(item._item ? item._item : item, renderer);
 				}
 			}
 		},
@@ -1436,11 +1506,11 @@ function(
 			// returns: Boolean
 			if (this._isEditing && this._edProps){
 				return item.id == this._edProps.editedItem.id;
-			}else{
-				return this.owner ?  
-					this.owner.isItemHovered(item) : 
-					this.hoveredItem != null && this.hoveredItem.id == item.id;
 			}
+			return this.owner ?  
+				this.owner.isItemHovered(item) : 
+				this.hoveredItem != null && this.hoveredItem.id == item.id;
+			
 		},
 		
 		isItemFocused: function(item){
@@ -1469,9 +1539,8 @@ function(
 		_getSelectionModeAttr: function(value){			
 			if(this.owner){
 				return this.owner.get("selectionMode");
-			}else{
-				return this.inherited(arguments);
 			}
+			return this.inherited(arguments);			
 		},
 		
 		_setSelectedItemAttr: function(value){			
@@ -1485,9 +1554,8 @@ function(
 		_getSelectedItemAttr: function(value){			
 			if(this.owner){
 				return this.owner.get("selectedItem");
-			}else{
-				return this.selectedItem; // no getter on super class (dojox.widget.Selection)
 			}
+			return this.selectedItem; // no getter on super class (dojox.widget.Selection)			
 		},
 		
 		_setSelectedItemsAttr: function(value){			
@@ -1501,17 +1569,15 @@ function(
 		_getSelectedItemsAttr: function(){			
 			if(this.owner){
 				return this.owner.get("selectedItems");
-			}else{
-				return this.inherited(arguments);
 			}
+			return this.inherited(arguments);
 		},
 		
 		isItemSelected: function(item){			
 			if(this.owner){
 				return this.owner.isItemSelected(item);
-			}else{
-				return this.inherited(arguments);
 			}
+			return this.inherited(arguments);
 		},
 		
 		selectFromEvent: function(e, item, renderer, dispatch){			
@@ -1541,12 +1607,13 @@ function(
 		createItemFunc: function(view, d, e){
 		 	// summary:
 			//		A user supplied function that creates a new event.
-			// view:
+			// view: ViewBase
 			//		the current view,
-			// d:
+			// d: Date
 			//		the date at the clicked location.
-			// e:
+			// e: MouseEvemt
 			//		the mouse event (can be used to return null for example)
+
 		},
 		=====*/
 
@@ -1554,9 +1621,8 @@ function(
 		_getCreateItemFuncAttr: function(){			
 			if(this.owner){
 				return this.owner.get("createItemFunc");
-			}else{
-				return this.createItemFunc;
 			}
+			return this.createItemFunc;
 		},
 		
 		// createOnGridClick: Boolean
@@ -1567,9 +1633,8 @@ function(
 		_getCreateOnGridClickAttr: function(){
 			if(this.owner){
 				return this.owner.get("createOnGridClick");
-			}else{
-				return this.createOnGridClick;
 			}
+			return this.createOnGridClick;			
 		},
 		
 		////////////////////////////////////////////////////////////////////
@@ -1578,7 +1643,9 @@ function(
 		//
 		///////////////////////////////////////////////////////////////////	
 		
-		_gridMouseDown: false,
+		_gridMouseDown: false,		
+		_tempIdCount: 0,
+		_tempItemsMap: null,
 				
 		_onGridMouseDown: function(e){
 			// tags:
@@ -1608,22 +1675,44 @@ function(
 					return;
 				}
 				
-				var newItem = f(this, this.getTime(e), e);
+				var newItem = this._createdEvent = f(this, this.getTime(e), e);
 								
 				var store = this.get("store");
 											
 				if(!newItem || store == null){
 					return;
 				}
+								
+				// calendar needs an ID to work with
+				if(store.getIdentity(newItem) == undefined){
+					var id = "_tempId_" + (this._tempIdCount++);
+					newItem[store.idProperty] = id;
+					if(this._tempItemsMap == null){
+						this._tempItemsMap = {};
+					}
+					this._tempItemsMap[id] = true;
+				}
+								
+				var newRenderItem = this.itemToRenderItem(newItem, store);				
+				newRenderItem._item = newItem;
+				this._setItemStoreState(newItem, "unstored");
 				
-				store.put(newItem);
+				// add the new temporary item to the displayed list and force view refresh
+				var owner = this._getTopOwner();
+				var items = owner.get("items");
 				
-				var renderers = this.getRenderers(newItem);
-				// renderer created when item put in store
+				owner.set("items", items ? items.concat([newRenderItem]) : [newRenderItem]);
+								
+				this._refreshItemsRendering();
+				
+				// renderer created in _refreshItemsRenderering()
+				var renderers = this.getRenderers(newItem);				
 				if(renderers && renderers.length>0){
 					var renderer = renderers[0];					
 					if(renderer){
+						// trigger editing
 						this._onRendererHandleMouseDown(e, renderer.renderer, "resizeEnd");
+						this._startItemEditing(newRenderItem, "mouse");	
 					}					
 				}
 			}
@@ -1647,7 +1736,7 @@ function(
 
 			this._gridProps = {
 				event: e,				
-				fromItem: this.isAscendantHasClass(e.target, this.eventContainer, "dojoxCalendarEventContainer")
+				fromItem: this.isAscendantHasClass(e.target, this.eventContainer, "dojoxCalendarEvent")
 			};			
 	
 			if(this._isEditing){
@@ -1679,7 +1768,7 @@ function(
 			//		private
 
 			if(obj && obj._isEditing){
-				p = obj._edProps;
+				var p = obj._edProps;
 				if(p && p.endEditingTimer){
 					clearTimeout(p.endEditingTimer);
 					p.endEditingTimer = null;
@@ -1841,7 +1930,7 @@ function(
 			var list = this.itemToRenderer[item.id];
 
 			if(list == null){
-				return;
+				return null;
 			}
 
 			// trivial and most common use case.
@@ -1903,8 +1992,8 @@ function(
 			//		The item represented by the renderer.
 			// rendererKind: String
 			//		The kind of renderer.
-			// returns: Boolean
-			return this.editable && (this.owner ? this.owner.isItemEditable() : true);
+			// returns: Boolean			
+			return this.getItemStoreState(item) != "storing" && this.editable && (this.owner ? this.owner.isItemEditable(item, rendererKind) : true);
 		},
 
 		isItemMoveEnabled: function(item, rendererKind){
@@ -1967,10 +2056,12 @@ function(
 			// tags:
 			//		protected
 
-			this._isEditing = true;			
+			this._isEditing = true;
+			this._getTopOwner()._isEditing = true;
 			var p = this._edProps;
 			
 			p.editedItem = item;
+			p.storeItem = item._item;
 			p.eventSource = eventSource;
 			
 			p.secItem = this._secondarySheet ? this._findRenderItem(item.id, this._secondarySheet.renderData.items) : null;
@@ -2014,6 +2105,7 @@ function(
 			
 			this._onItemEditBegin({
 				item: item,
+				storeItem: p.storeItem,
 				eventSource: eventSource
 			});
 		},
@@ -2046,7 +2138,8 @@ function(
 			// tags:
 			//		protected
 
-			this._isEditing = false;					
+			this._isEditing = false;
+			this._getTopOwner()._isEditing = false;
 			
 			var p = this._edProps;
 			
@@ -2059,11 +2152,10 @@ function(
 				this.rendererList = p.editRendererList.concat(this.rendererList);
 				lang.mixin(this.itemToRenderer, p.editItemToRenderer);
 			}
-			
-			var store = this.get("store");
-						
+
 			this._onItemEditEnd(lang.mixin(this._createItemEditEvent(), {
-				item: this.renderItemToItem(p.editedItem, store),
+				item: p.editedItem,
+				storeItem: p.storeItem,
 				eventSource: eventSource,
 				completed: !canceled
 			}));
@@ -2080,18 +2172,83 @@ function(
 			this._dispatchCalendarEvt(e, "onItemEditEnd");
 			
 			if(!e.isDefaultPrevented()){
-				if(e.completed){
+				
+				var store = this.get("store");
+				
+				// updated store item
+				var storeItem = this.renderItemToItem(e.item, store);
+				
+				var s = this._getItemStoreStateObj(e.item);
+				
+				if(s != null && s.state == "unstored"){
+														
+					if(e.completed){
+						// renderItemToItem cannot find the original data item
+						// (as it does not exist in the store yet) to mixin with.
+						// so we must do it here.
+						storeItem = lang.mixin(s.item, storeItem);
+						this._setItemStoreState(storeItem, "storing");
+						var oldID = store.getIdentity(storeItem);
+						var options = null;
+						
+						if(this._tempItemsMap && this._tempItemsMap[oldID]){
+							options = {temporaryId: oldID}; 
+							delete this._tempItemsMap[oldID];
+							delete storeItem[store.idProperty];							
+						}
+						
+						// add to the store.
+						when(store.add(storeItem, options), lang.hitch(this, function(res){
+							var id;
+							if(lang.isObject(res)){
+								id = store.getIdentity(res);
+							}else{
+								id = res;
+							}
+							
+							if(id != oldID){							
+								this._removeRenderItem(oldID);
+							}
+						}));
+						
+					}else{ // creation canceled
+						// cleanup items list
+						
+						this._removeRenderItem(s.id);											
+					}									
+					
+				} else if(e.completed){
 					// Inject new properties in data store item				
-					// and apply data changes
-					var store = this.get("store");
-					store.put(e.item, store);
-				}else{			
+					// and apply data changes		
+					this._setItemStoreState(storeItem, "storing");
+					store.put(storeItem);								
+				}else{
 					e.item.startTime = this._editStartTimeSave; 
 					e.item.endTime = this._editEndTimeSave;
 				}
 			}
 		},
 		
+		_removeRenderItem: function(id){
+			
+			var owner = this._getTopOwner();
+			var items = owner.get("items");
+			var l = items.length; 
+			var found = false;
+			for(var i=l-1; i>=0; i--){
+				if(items[i].id == id){
+					items.splice(i, 1);
+					found = true;
+					break;
+				}
+			}
+			this._cleanItemStoreState(id);
+			if(found){
+				owner.set("items", items); //force a complete relayout	
+				this.invalidateLayout();
+			}
+		},
+		
 		onItemEditEnd: function(e){
 			// summary:
 			//		Event dispatched when the item is leaving the editing mode.
@@ -2150,6 +2307,7 @@ function(
 			
 			this._onItemEditBeginGesture(this.__fixEvt(lang.mixin(this._createItemEditEvent(), {
 				item: item,
+				storeItem: p.storeItem,
 				startTime: item.startTime,
 				endTime: item.endTime,
 				editKind: editKind,
@@ -2392,6 +2550,7 @@ function(
 
 			var evt = lang.mixin(this._createItemEditEvent(), {
 				item: item,
+				storeItem: p.storeItem,
 				startTime: item.startTime,
 				endTime: item.endTime,
 				editKind: editKind,
@@ -2410,7 +2569,7 @@ function(
 			// prevent invalid range
 			if(cal.compare(item.startTime, item.endTime) == 1){
 				var tmp = item.startTime;
-				item.startTime = item.startTime;
+				item.startTime = item.endTime;
 				item.endTime = tmp;
 			}
 			
@@ -2601,6 +2760,7 @@ function(
 			
 			this._onItemEditEndGesture(lang.mixin(this._createItemEditEvent(), {
 				item: item,
+				storeItem: p.storeItem,
 				startTime: item.startTime,
 				endTime: item.endTime,
 				editKind: p.editKind,
